local ADDON_NAME = 'Gemcount'
local ADDON_LOCALE_NAME = ADDON_NAME
local DATAOBJECT_NAME = ADDON_NAME
local ADDON_DB = ADDON_NAME..'DB'
--- Event for first Init with Gemdata avaiable, Find some Usefull
-- don't: VARIABLES_LOADED, PLAYER_AVG_ITEM_LEVEL_READY
-- do: TIME_PLAYED_MSG, WORLD_MAP_UPDATE
local INIT_EVENT = 'TIME_PLAYED_MSG'


-- Faster access to global environment
local _G = _G
-- Use local variable instead of upvalue
local LibStub = _G.LibStub
local AceAddon = LibStub:GetLibrary('AceAddon-3.0')
local ldb      = LibStub:GetLibrary('LibDataBroker-1.1')
local LibQTip  = LibStub:GetLibrary('LibQTip-1.0')
local L        = LibStub:GetLibrary('AceLocale-3.0'):GetLocale(ADDON_LOCALE_NAME)
local AceDB    = LibStub:GetLibrary('AceDB-3.0')


--[==[
--  Create addon table and databroker template
--]==]
--- our addon object
local addon = AceAddon:NewAddon(ADDON_NAME, 'AceEvent-3.0')

--- a template for the dataobject
local dataObjectTemplate = {
    type = 'data source',
    text = "Gemcount",
    icon = "Interface\\Icons\\inv_misc_cutgemsuperior6",
}

--- default values for our options
local defaults = {
    profile = {
        listr = false,
        listy = true,
        listb = false,
        listm = false
    }
}




--[==[
--  Debugging related stuff.
--]==]
do
    local libDebugLog = LibStub:GetLibrary('LibDebugLog-1.0',true)
    if libDebugLog then
        libDebugLog:Embed(addon)
    else
        function addon:Debug (...)
        end
    end
end

--[==[
--  Declaration of some constants
--]==]

-- Some color stuff
local RED_FONT_COLOR           = "|cffff0000"
local YELLOW_FONT_COLOR        = "|cffffff00"
local BLUE_FONT_COLOR          = "|cff0070dd"
local META_FONT_COLOR          = "|cffa6a6a6"
local META_ACTIVE_FONT_COLOR   = "|cff87cefa"
local META_INACTIVE_FONT_COLOR = "|cffa6a6a6"
-- End of color formated text
local END_COLOR = "|r"


local slots={
    "HeadSlot",
    "NeckSlot",
    "ShoulderSlot",
    "BackSlot",
    "ChestSlot",
    "ShirtSlot",
    "TabardSlot",
    "WristSlot",
    "HandsSlot",
    "WaistSlot",
    "LegsSlot",
    "FeetSlot",
    "Finger0Slot",
    "Finger1Slot",
    "Trinket0Slot",
    "Trinket1Slot",
    "MainHandSlot",
    "SecondaryHandSlot",
    "RangedSlot",
    "AmmoSlot",
}

local gemColorLocalized = {}
do
    local cGL = gemColorLocalized
    cGL.red, cGL.blue, cGL.yellow, cGL.purple, cGL.green, cGL.orange, cGL.meta, _, cGL.prismatic, _ = GetAuctionItemSubClasses(8)
end

--[==[
--  Begin of local functions
--]==]

local tipFrame
local checkMetaTooltip
--- checks whether the given meta gem ID is active or not
-- @param gemlink the item link of a gem,
-- @return boolean true/false which is the active state of the given item ID of an metagem
local function checkMeta(gemlink)
    -- Checks if Meta is active
    local tooltipName = ADDON_NAME..'MetaTooltip'
    checkMetaTooltip = checkMetaTooltip or CreateFrame('GameTooltip', tooltipName, UIParent, 'GameTooltipTemplate')
    checkMetaTooltip:SetOwner(UIParent, 'ANCHOR_NONE')
    checkMetaTooltip:SetHyperlink(gemlink)
    local metaLine = checkMetaTooltip:NumLines() - 2
    local toolText = getglobal(tooltipName..'TextLeft'..metaLine)
    local reqText = toolText:GetText()
    checkMetaTooltip:Hide()
    if( string.match(reqText, "\124cff808080") ) then
        META_FONT_COLOR = META_INACTIVE_FONT_COLOR
        return false
    else
        META_FONT_COLOR = META_ACTIVE_FONT_COLOR
        return true
    end
end

--- Count the number of gems the player has.
-- @return redgems, yellowgems, bluegems, metagems, metaactive where the first four are the amount
--         of equipped gems of the given color. Note that dual colored/
--         prismatic gems count for each color.
--         The last tells whether the meta gem is active.
local function countGems()
    local redgems = 0
    local yellowgems = 0
    local bluegems = 0
    local metagems = 0
    local metaactive = false
    addon.gemTable = addon.gemTable or {}
    for _, slotname in pairs(slots) do
        local itemlink = GetInventoryItemLink('player', GetInventorySlotInfo(slotname))
        if itemlink then
            for i = 1,3 do
                local itemGemId = slotname .. i
                local gem=select(2,GetItemGem(itemlink,i))
                if gem then
                    local gemname, gemlink, _, _, _, gemtype, gemcolor = GetItemInfo(gem)
                    if gemcolor == gemColorLocalized.purple then
                        redgems = redgems + 1
                        bluegems = bluegems + 1
                    elseif gemcolor == gemColorLocalized.green then
                        yellowgems = yellowgems + 1
                        bluegems = bluegems + 1
                    elseif gemcolor == gemColorLocalized.orange then
                        yellowgems = yellowgems + 1
                        redgems = redgems + 1
                    elseif gemcolor == gemColorLocalized.prismatic then
                        redgems = redgems + 1
                        bluegems = bluegems + 1
                        yellowgems = yellowgems + 1
                    elseif gemcolor == gemColorLocalized.red then
                        redgems = redgems + 1
                    elseif gemcolor == gemColorLocalized.yellow then
                        yellowgems = yellowgems + 1
                    elseif gemcolor == gemColorLocalized.blue then
                        bluegems = bluegems + 1
                    elseif gemcolor == gemColorLocalized.meta then
                        metagems = metagems + 1
                        metaactive =  checkMeta(gemlink)
                    else
                        addon:Debug("Gem color '" .. gemcolor .. "' is not known. Probably a localization problem.")
                    end
                    addon.gemTable[itemGemId] = {name = gemname, link = gemlink, colorLocalized = gemcolor}
                else -- no gem
                    addon.gemTable[itemGemId] = nil
                end
            end
        end
    end
    addon.redgems    = redgems
    addon.yellowgems = yellowgems
    addon.bluegems   = bluegems
    addon.metagems   = metagems
    addon.metaactive = metaactive
end

-- update the ldb Text
local function updateLDB()
    countGems()
    local tmptext =      " " .. RED_FONT_COLOR    .. addon.redgems    .. END_COLOR
    tmptext = tmptext .. " " .. YELLOW_FONT_COLOR .. addon.yellowgems .. END_COLOR
    tmptext = tmptext .. " " .. BLUE_FONT_COLOR   .. addon.bluegems   .. END_COLOR
    tmptext = tmptext .. " " .. META_FONT_COLOR .. addon.metagems .. END_COLOR
    addon.dataObject.text = tmptext
end



--         Show details of Gem in Default Game Tooltip
-- @param _ don'need to know the cell where the event comes from
-- @param link the item id of an Item,
-- @param _ don'need to know the button
local function showdetails(_,link,_)
    tipFrame = tipFrame or GameTooltip
    GameTooltip_SetDefaultAnchor(tipFrame, UIParent)
    tipFrame:SetHyperlink(link)
    tipFrame:Show()
end

--         Hide the Tooltip from above
-- @param _ don'need to know the cell where the event comes from
-- @param link the item id of an Item,
-- @param _ don'need to know the button
local function hidedetails(_,link,_)
    tipFrame:FadeOut()
end

-- Paste the clicked gem into Chatframe
-- @param _ don'need to know the cell where the event comes from
-- @param link the item id of an Item,
-- @param _ don'need to know the button
local function TooltipClick(_,link,_)
    if ChatFrame1EditBox:IsShown() then
        ChatFrame1EditBox:Insert(link)
    end
end

--[==[
--  End of local functions
--]==]

--- Callback for most registered Ace3 Events.
function addon:ReCalc()
    updateLDB()
end

--- Callback for the Initial event, used only once.
function addon:firstCalc()
    -- we need this only one time for init
    addon:UnregisterEvent(INIT_EVENT)
    updateLDB()
end


--- Called when the addon is enabled.
-- Useful for registering to events, setting things up.
function addon:OnEnable()
    addon.db = AceDB:New(ADDON_DB, defaults)
    addon:RegisterEvent('PLAYER_EQUIPMENT_CHANGED', 'ReCalc')
    addon:RegisterEvent('SOCKET_INFO_UPDATE',       'ReCalc')
    addon:RegisterEvent(INIT_EVENT, 'firstCalc')
    addon.dataObject = ldb:NewDataObject(DATAOBJECT_NAME, dataObjectTemplate)
end

-- toggles expand/collapse for the list of given gem color in tooltip
--@param cell - cell where the event comes from
--@param color - color of gem
--@param button - pressed button
local function togglelist(cell,color,button)
    if color == "red" then
        addon.db.profile.listr = not addon.db.profile.listr
    elseif color == "yellow" then
        addon.db.profile.listy = not addon.db.profile.listy
    elseif color == "blue" then
        addon.db.profile.listb = not addon.db.profile.listb
    elseif color == "meta" then
        addon.db.profile.listm = not addon.db.profile.listm
    end
    dataObjectTemplate:OnEnter()
end


--- Fill the tooltip with information
-- Does not clear the tooltip.
-- @param tip is a tooltip
local function fillTooltip(tip)
    tip:AddHeader("",ADDON_NAME)
    --tip:AddSeperator

    if addon.db.profile.listr then
        local line = tip:AddLine("-",RED_FONT_COLOR .. gemColorLocalized.red .. ": " .. END_COLOR, addon.redgems)
        tip:SetCellScript(line, 2, "OnMouseUp", togglelist, "red")
        for slotname,data in pairs(addon.gemTable) do
            if (data.colorLocalized == gemColorLocalized.purple or data.colorLocalized == gemColorLocalized.orange or data.colorLocalized == gemColorLocalized.prismatic or data.colorLocalized == gemColorLocalized.red) then
                local link = data.link
                local line = tip:AddLine("",data.link, data.colorLocalized, slotname)
                tip:SetCellScript(line, 2, "OnEnter", showdetails, link)
                tip:SetCellScript(line, 2, "OnMouseUp", TooltipClick, link)
                tip:SetCellScript(line, 2, "OnLeave", hidedetails)
                -- maybe show the Item in wich the Gem on this Line ist ?
                -- tip:SetCellScript(line, 3, "OnEnter", showdetails, ????)
            end
        end
    else
        local line = tip:AddLine("+",RED_FONT_COLOR .. gemColorLocalized.red .. ": " .. END_COLOR, addon.redgems)
        tip:SetCellScript(line, 2, "OnMouseUp", togglelist, "red")
    end


    if addon.db.profile.listy then
        local line = tip:AddLine("-",YELLOW_FONT_COLOR  .. gemColorLocalized.yellow .. ": "..END_COLOR,addon.yellowgems)
        tip:SetCellScript(line, 2, "OnMouseUp", togglelist, "yellow")
        for slotname,data in pairs(addon.gemTable) do
            if (data.colorLocalized == gemColorLocalized.green or data.colorLocalized == gemColorLocalized.orange or data.colorLocalized == gemColorLocalized.prismatic or data.colorLocalized == gemColorLocalized.yellow) then
                local link = data.link
                local line = tip:AddLine("",data.link, data.colorLocalized, slotname)
                tip:SetCellScript(line, 2, "OnEnter", showdetails, link)
                tip:SetCellScript(line, 2, "OnMouseUp", TooltipClick, link)
                tip:SetCellScript(line, 2, "OnLeave", hidedetails)
                -- maybe show the Item in wich the Gem on this Line ist ?
                -- tip:SetCellScript(line, 3, "OnEnter", showdetails, ????)
            end
        end
    else
        local line = tip:AddLine("+",YELLOW_FONT_COLOR  .. gemColorLocalized.yellow .. ": "..END_COLOR,addon.yellowgems)
        tip:SetCellScript(line, 2, "OnMouseUp", togglelist, "yellow")
    end

    if addon.db.profile.listb then
        local line = tip:AddLine("-",BLUE_FONT_COLOR .. gemColorLocalized.blue .. ": "..END_COLOR,addon.bluegems)
        tip:SetCellScript(line, 2, "OnMouseUp", togglelist, "blue")
        for slotname,data in pairs(addon.gemTable) do
            if (data.colorLocalized == gemColorLocalized.purple or data.colorLocalized == gemColorLocalized.green or data.colorLocalized == gemColorLocalized.prismatic or data.colorLocalized == gemColorLocalized.blue) then
                local link = data.link
                local line = tip:AddLine("",data.link, data.colorLocalized, slotname)
                tip:SetCellScript(line, 2, "OnEnter", showdetails, link)
                tip:SetCellScript(line, 2, "OnMouseUp", TooltipClick, link)
                tip:SetCellScript(line, 2, "OnLeave", hidedetails)
                -- maybe show the Item in wich the Gem on this Line ist ?
                -- tip:SetCellScript(line, 3, "OnEnter", showdetails, ????)
            end
        end
    else
        local line = tip:AddLine("+",BLUE_FONT_COLOR .. gemColorLocalized.blue .. ": "..END_COLOR,addon.bluegems)
        tip:SetCellScript(line, 2, "OnMouseUp", togglelist, "blue")
    end

    if addon.db.profile.listm then
        local line = tip:AddLine("-",META_FONT_COLOR .. gemColorLocalized.meta .. ": "..END_COLOR,addon.metagems)
        tip:SetCellScript(line, 2, "OnMouseUp", togglelist, "meta")
        for slotname,data in pairs(addon.gemTable) do
            if (data.colorLocalized == gemColorLocalized.meta) then
                local link = data.link
                local line = tip:AddLine("",data.link, data.colorLocalized, slotname)
                tip:SetCellScript(line, 2, "OnEnter", showdetails, link)
                tip:SetCellScript(line, 2, "OnMouseUp", TooltipClick, link)
                tip:SetCellScript(line, 2, "OnLeave", hidedetails)
                -- maybe show the Item in wich the Gem on this Line ist ?
                -- tip:SetCellScript(line, 3, "OnEnter", showdetails, ????)
            end
        end
    else
        local line = tip:AddLine("+",META_FONT_COLOR .. gemColorLocalized.meta .. ": "..END_COLOR,addon.metagems)
        tip:SetCellScript(line, 2, "OnMouseUp", togglelist, "meta")
    end
end

--- release the tooltip so the frame could be recylcled
local function releasetooltip()
    if MouseIsOver(addon.tooltip) then return end
    addon.tooltip:SetScript("OnLeave", nil)
    LibQTip:Release(addon.tooltip)
    addon.tooltip = nil
end


--- Callback of LDB.
-- Called when the tooltip is shown. Used to add information.
function dataObjectTemplate:OnEnter()
    if addon.tooltip and LibQTip:IsAcquired("Broker_" .. DATAOBJECT_NAME) then
        addon:Debug("Reuse")
        addon.tooltip:Clear()
    else
        addon:Debug("Acquire")
        addon.tooltip = LibQTip:Acquire("Broker_" .. DATAOBJECT_NAME, 4, "LEFT", "LEFT", "CENTER", "RIGHT")
        addon.tooltip:SmartAnchorTo(self)
        addon.tooltip:SetAutoHideDelay(0.25, self)
    end
    fillTooltip(addon.tooltip)
    addon.tooltip:SetScript("OnLeave", releasetooltip)
    addon.tooltip:Show()
end

--- Callback of LDB, called when the mouse leaves the dataObject
function dataObjectTemplate:OnLeave()
    releasetooltip()
end

--- Callback of LDB, called when the dataObject is clicked
-- @param button the pressed button
function dataObjectTemplate:OnClick(button, ...)
    if button == 'LeftButton' then
        ToggleCharacter('PaperDollFrame')
    end
end
